/*
 *	File:		Application.r
 *
 *	Contains:	General resources for CPlus Tool Template based tools.
 *
 *	Written by:	Rick Violet and Chad Williams
 *
 *	Copyright:	 1992-1994 by Apple Computer, Inc., all rights reserved.
 *
 *	Change History (most recent first):
 *
 *		 <2>	 1/14/94	CMW		Update for Apple Scriptable external tools.
 *				11/18/92	RV		New file
 *
 *	To Do:
 */

#include "SysTypes.r"
#include "Types.r"

#include "AEUserTermTypes.r"

#include "Application.R.h"

/**/
/*	Version Resource																*/
/**/
resource 'vers' (1) 
{
	0x01, 0x01, release, 0x00,
	verUS,
	"1.1",
	"1.1, Copyright  1989-1994 Apple Computer, Inc."
};

/**/
/*	MBAR	-	determines what menus are loaded									*/			
/**/
resource 'MBAR' ( kMenuBarID, preload ) 
{
	{ kAppleMenuID, kFileMenuID };
};


/**/
/*	MENU	-	apple menu															*/			
/**/
resource 'MENU' ( kAppleMenuID, preload ) 
{
	kAppleMenuID, textMenuProc,
	0b1111111111111111111111111111101,	/* enable About & DAs, disable separator */
	enabled, apple,
	{	
		"About ExTool",	noIcon, 	noKey, 	noMark, 	plain,
		"-", 				noIcon, 	noKey, 	noMark, 	plain
	}
};

/**/
/*	MENU	-	file menu															*/			
/**/
resource 'MENU' (kFileMenuID, preload) 
{
	kFileMenuID, textMenuProc,
	0b0000000000000000000000000000001,	/* enable Quit only, program enables others */
	enabled, "File",
	{
		"Quit",					noicon,		"Q", 	nomark, 	plain
	}
};

/**/
/*	ALRT	-	About box alert														*/
/**/
resource 'ALRT' (kAboutAlertID, purgeable) 
{
	{40, 20, 160, 330 }, kAboutAlertID, 
	{
		OK, visible, silent;
		OK, visible, silent;
		OK, visible, silent;
		OK, visible, silent
	};
};

/**/
/*	DITL	-	About box alert	item list											*/
/**/
resource 'DITL' ( kAboutAlertID, purgeable ) 
{ {								
	{ 88, 184, 108, 264}, Button	 { enabled,	 "OK"	},
	{  8,   8,  26, 245}, StaticText { disabled, "Example Virtual User External Tool."	},
	{ 32,   8,  48, 244}, StaticText { disabled, "Written in C++."	},
	{ 56,   8,  71, 244}, StaticText { disabled, "Brought to you by: Rick Violet"	},
	{ 80,   8, 114, 170}, StaticText { disabled, "Copyright  1992-94\n"
												 "Apple Computer, Inc."	}
} };


/**/
/*	SIZE																			*/
/**/
resource 'SIZE' (-1) 
{
	reserved,
	acceptSuspendResumeEvents,
	reserved,
	canBackground,
	multiFinderAware,
	backgroundAndForeground,
	dontGetFrontClicks,
	ignoreChildDiedEvents,
	is32BitCompatible,
	isHighLevelEventAware,
	localAndRemoteHLEvents,
	notStationeryAware,
	dontUseTextEditServices,
	reserved, 
	reserved, 
	reserved,
	kPrefSize * 1024,
	kMinSize * 1024
};


/**/
/*	BNDL																			*/
/**/
resource 'BNDL' (128) 
{
	kOwnerSignature, 0,
	{
		'ICN#', { 0, 128 },
		'FREF', { 0, 128 }
	}
};

/**/
/*	Creator Signature String														*/
/**/
type kOwnerSignature as 'STR ';
resource kOwnerSignature (0) 
{
	"C++ External Tool Application for use with Virtual User."
};

/**/
/*	FREF																			*/
/**/
resource 'FREF' (128) 
{
	'APPL', 0, ""
};


/**/
/*	ICN#																			*/
/**/
resource 'ICN#' (128) {
	{ /* array: 2 elements */
		/* [1] */
		$"04 30 40 00 0A 50 A0 00 0B 91 10 02 08 22 08 03"
		$"12 24 04 05 20 28 02 09 40 10 01 11 80 0C 00 A1"
		$"80 03 FF C2 7E 00 FF 04 01 00 7F 04 03 00 1E 08"
		$"04 E0 00 0C 08 E0 00 0A 10 E0 00 09 08 C0 00 06"
		$"04 87 FE 04 02 88 01 04 01 88 00 84 00 88 00 44"
		$"00 88 00 44 00 88 00 C4 01 10 01 88 02 28 03 10"
		$"01 C4 04 E0 00 02 08 00 73 BF FB EE 4C A2 8A 2A"
		$"40 AA AA EA 52 AA AA 24 5E A2 8A EA 73 BE FB 8E",
		/* [2] */
		$"04 30 40 00 0E 70 E0 00 0F F1 F0 02 0F E3 F8 03"
		$"1F E7 FC 07 3F EF FE 0F 7F FF FF 1F FF FF FF BF"
		$"FF FF FF FE 7F FF FF FC 01 FF FF FC 03 FF FF F8"
		$"07 FF FF FC 0F FF FF FE 1F FF FF FF 0F FF FF FE"
		$"07 FF FF FC 03 FF FF FC 01 FF FF FC 00 FF FF FC"
		$"00 FF FF FC 00 FF FF FC 01 FF FF F8 03 EF FF F0"
		$"01 C7 FC E0 00 03 F8 00 73 BF FB EE 7F BE FB EE"
		$"7F BE FB EE 7F BE FB E4 7F BE FB EE 73 BE FB 8E"
	}
};

/**/
/*	ALRT	-	Standard Error Message alert										*/
/**/
resource 'ALRT' (kUserAlertID, purgeable) 
{
	{40, 20, 157, 372},
	kUserAlertID,
	{ 	
		OK, visible, silent,
		OK, visible, silent,
		OK, visible, silent,
		OK, visible, silent
	}
};

/**/
/*	DITL	-	Standard Error Message alert items									*/
/**/
resource 'DITL' (kUserAlertID, purgeable) 
{ {								/* 3 Items */
/* [  1] */ 	{   80,   141,   100,   221},	Button		{ enabled,	"OK"	},
/* [  2] */ 	{   10,    52,    70,   352},	StaticText	{ disabled,	"Error:\n"
																		"^0"	},
/* [  3] */ 	{   10,    10,    42,    42},	Icon		{ disabled,	2	}
} };


/**/
/*	STR#	-	Application Messages												*/
/**/
resource 'STR#' ( kAppErrStrings, purgeable ) 
{
	{
	"This application runs only on a Macintosh Plus or later machine.";
	"The memory partition is too small.\nUse Finders Get Info to increase the size.";
	"This application runs only on System Software version 7.0 or greater.";
	}
};


/**/
/*	aete	-	Apple Script dictionary resource									*/
/*				Defines one service: "DoService"									*/
/**/
resource 'aete' (0) {
	0x1,
	0x0,
	english,
	roman,
	{	/* array Suites: 1 elements */
		/* [1] */
		"VU external tool",
		"",
		'v.u.',
		1,
		1,
		{	/* array Events: 1 elements */
			/* [1] */
			"DoService",
			"Do an external tool service",
			'v.u.',
			'extc',
			'****',
			"",
			replyRequired,
			singleItem,
			notEnumerated,
			notTightBindingFunction,
			reserved, reserved, reserved, reserved, reserved,
			reserved, reserved, reserved,
			verbEvent,
			reserved, reserved, reserved,
			'TEXT',
			"",
			directParamRequired,
			singleItem,
			notEnumerated,
			doesntChangeState,
			reserved, reserved, reserved, reserved, reserved,
			reserved, reserved, reserved, reserved, reserved,
			reserved, reserved,
			{	/* array OtherParams: 1 elements */
				/* [1] */
				"Parameters",
				'extp',
				'list',
				"",
				required,
				singleItem,
				notEnumerated,
				reserved, reserved, reserved, reserved, reserved,
				reserved, reserved, reserved, reserved,
				prepositionParam,
				notFeminine,
				notMasculine,
				singular
			}
		},
		{	/* array Classes: 0 elements */
		},
		{	/* array ComparisonOps: 0 elements */
		},
		{	/* array Enumerations: 0 elements */
		}
	}
};

/**/
/*	CURS	-	Beach Ball 1														*/
/**/
resource 'CURS' (128, preload) 
{
	$"07C0 1F30 3F08 7F04 7F04 FF02 FF02 FFFE"
	$"81FE 81FE 41FC 41FC 21F8 19F0 07C0",
	$"07C0 1FF0 3FF8 7FFC 7FFC FFFE FFFE FFFE"
	$"FFFE FFFE 7FFC 7FFC 3FF8 1FF0 07C0",
	{7, 7}
};

/**/
/*	CURS	-	Beach Ball 2														*/
/**/
resource 'CURS' (129, preload) 
{
	$"07C0 1FF0 3FF8 5FF4 4FE4 87C2 8382 8102"
	$"8382 87C2 4FE4 5FF4 3FF8 1FF0 07C0",
	$"07C0 1FF0 3FF8 7FFC 7FFC FFFE FFFE FFFE"
	$"FFFE FFFE 7FFC 7FFC 3FF8 1FF0 07C0",
	{7, 7}
};

/**/
/*	CURS	-	Beach Ball 3														*/
/**/
resource 'CURS' (130, preload) 
{
	$"07C0 19F0 21F8 41FC 41FC 81FE 81FE FFFE"
	$"FF02 FF02 7F04 7F04 3F08 1F30 07C0",
	$"07C0 1FF0 3FF8 7FFC 7FFC FFFE FFFE FFFE"
	$"FFFE FFFE 7FFC 7FFC 3FF8 1FF0 07C0",
	{7, 7}
};

/**/
/*	CURS	-	Beach Ball 3														*/
/**/
resource 'CURS' (131, preload) 
{
	$"07C0 1830 2008 701C 783C FC7E FEFE FFFE"
	$"FEFE FC7E 783C 701C 2008 1830 07C0",
	$"07C0 1FF0 3FF8 7FFC 7FFC FFFE FFFE FFFE"
	$"FFFE FFFE 7FFC 7FFC 3FF8 1FF0 07C0",
	{7, 7}
};

/**/
/*	acur	-	acur for Beach Ball													*/
/**/
resource 'acur' (0, preload) 
{
	{	
		128,
		129,
		130,
		131
	}
};

